import requests
import urllib.parse
import csv
import os
import re

AUTH_URL = 'https://app.leak.center/uaa/oauth/token'
LEAKS_ENDPOINT = "https://app.leak.center/svc-saas/leak/search"
CLIENT_ID = 'client-api'
CLIENT_SECRET = 'comfy-litigate-embargo-forelimb'

class Auth:
    username = None
    password = None

def display_help():
    print("""
    Welcome to the Leak Center Query Tool.
    
    Optional Files:
    1. auth.txt: Provide a text file with your login credentials in the format:
       username,password (e.g., john@doe.com,password123)
    2. input.txt: Provide the target domain (e.g., example.com but no subdomains like www).
    3. DateRange.txt: Specify a date range in US date format as yyyy-mm-dd TO yyyy-mm-dd or yyyy-mm-dd TO *.

    If these files are not provided, you will be prompted to input the required details interactively.
    """)

def get_credentials():
    if Auth.username and Auth.password:
        return Auth.username, Auth.password
    if os.path.exists("auth.txt"):
        with open("auth.txt", "r") as f:
            return f.read().strip().split(',')
    else:
        print("auth.txt not found.")
        Auth.username = input("Enter username: ")
        Auth.password = input("Enter password: ")
        return Auth.username, Auth.password

def get_access_token(username, password):
    data = {
        'grant_type': 'password',
        'username': username,
        'password': password,
        'client_id': CLIENT_ID,
        'client_secret': CLIENT_SECRET
    }
    try:
        response = requests.post(AUTH_URL, data=data)
        response.raise_for_status()
        token = response.json().get('access_token')
        if not token:
            raise ValueError("No access token in response.")
        print(f"Fetched token: {token[:10]}...")
        return token
    except requests.exceptions.RequestException as e:
        print("Failed to authenticate. Please check your credentials.")
        print(f"Error: {e}")
        exit(1)

def get_query_input():
    if os.path.exists("input.txt"):
        with open("input.txt", "r") as f:
            return f.read().strip()
    else:
        print("input.txt not found.")
        return input("Enter target domain (e.g., example.com but no subdomain): ").strip()

def parse_date_range_input(user_input):
    # Normalize user input into yyyy-mm-dd format
    formats = [
        r"(\d{4})[-.](\d{2})[-.](\d{2})",  # yyyy.mm.dd or yyyy-mm-dd
        r"(\d{2})[-.](\d{2})[-.](\d{2})"   # yy.mm.dd
    ]
    def normalize_date(date):
        if len(date[0]) == 2:  # Handle yy to yyyy
            date = ("20" + date[0],) + date[1:]
        return "-".join(date)

    match = None
    for fmt in formats:
        match = re.findall(fmt, user_input)
        if match:
            break

    if not match or len(match) != 2:
        print("Invalid date range format. Please use US date format (e.g., yyyy-mm-dd TO yyyy-mm-dd or yyyy.mm.dd TO *).")
        exit(1)

    start_date = normalize_date(match[0])
    end_date = normalize_date(match[1]) if match[1] != ('*',) else '*'
    return f"{start_date} TO {end_date}"

def get_date_range():
    if os.path.exists("DateRange.txt"):
        with open("DateRange.txt", "r") as f:
            return f.read().strip()
    else:
        print("DateRange.txt not found.")
        user_input = input("Enter date range (e.g., yyyy-mm-dd TO yyyy-mm-dd or yyyy.mm.dd TO *, or press Enter to skip): ").strip()
        if not user_input:
            return ""
        return parse_date_range_input(user_input)

def query_and_process_leaks(domain, token, date_range):
    page = 0
    with open("Leaks.csv", mode='w', newline='') as outfile:
        csv_writer = csv.writer(outfile, delimiter=',', quotechar='"', quoting=csv.QUOTE_MINIMAL)
        headers = ['id', 'createdAt', 'content', 'fileName', 'fileExtension', 'leakId', 'leakName',
                   'leakSource', 'leakSize', 'leakTags', 'leakPublishDate', 'leakDiscoverDate', 'cvssScore']
        csv_writer.writerow(headers)
        while True:
            params = {
                "page": page,
                "size": 100,
                "sort": "createdAt,desc",
                "query": domain + (" AND createdAt:[" + date_range + "]" if date_range else "")
            }

            headers_request = {"Authorization": f"Bearer {token}"}
            response = requests.get(LEAKS_ENDPOINT, params=params, headers=headers_request)
            response.raise_for_status()
            data = response.json()

            if not data['content']:
                print("No more content available.")
                break

            for item in data['content']:
                row = [item.get(header, '') for header in headers]
                csv_writer.writerow(row)

            print(f"Page {page} processed, total entries collected: {len(data['content'])}")

            if data.get('last', False):
                print("Last page of data reached.")
                break
            page += 1

def main():
    display_help()
    username, password = get_credentials()
    token = get_access_token(username, password)
    domain = get_query_input()
    date_range = get_date_range()
    query_and_process_leaks(domain, token, date_range)
    print("Completed. Results saved to Leaks.csv.")

if __name__ == "__main__":
    main()
